""" Custom error types """


class PEPDatabaseAgentError(Exception):
    """Base error type for pepdbagent custom errors."""

    def __init__(self, msg):
        super(PEPDatabaseAgentError, self).__init__(msg)


class SchemaError(PEPDatabaseAgentError):
    def __init__(self):
        super().__init__("""PEP_db connection error! The schema of connected db is incorrect""")


class RegistryPathError(PEPDatabaseAgentError):
    def __init__(self, msg=""):
        super().__init__(f"""Provided registry path is incorrect. {msg}""")


class ProjectNotFoundError(PEPDatabaseAgentError):
    def __init__(self, msg=""):
        super().__init__(f"""Project does not exist. {msg}""")


class ProjectUniqueNameError(PEPDatabaseAgentError):
    def __init__(self, msg=""):
        super().__init__(f"""{msg}""")


class IncorrectDateFormat(PEPDatabaseAgentError):
    def __init__(self, msg=""):
        super().__init__(f"""Incorrect date format was provided. {msg}""")


class FilterError(PEPDatabaseAgentError):
    def __init__(self, msg=""):
        super().__init__(f"""pepdbagent filter error. {msg}""")


class ProjectNotInFavorites(PEPDatabaseAgentError):
    """
    Project doesn't exist in favorites
    """

    def __init__(self, msg=""):
        super().__init__(f"""Project is not in favorites list. {msg}""")


class ProjectAlreadyInFavorites(PEPDatabaseAgentError):
    """
    Project doesn't exist in favorites
    """

    def __init__(self, msg=""):
        super().__init__(f"""Project is already in favorites list. {msg}""")


class SampleNotFoundError(PEPDatabaseAgentError):
    def __init__(self, msg=""):
        super().__init__(f"""Sample does not exist. {msg}""")


class ViewNotFoundError(PEPDatabaseAgentError):
    def __init__(self, msg=""):
        super().__init__(f"""View does not exist. {msg}""")


class SampleAlreadyInView(PEPDatabaseAgentError):
    """
    Sample is already in the view exception
    """

    def __init__(self, msg=""):
        super().__init__(f"""Sample is already in the view. {msg}""")
